package com.brilliance.isc.transfer.transaction;

import com.brilliance.isc.bo.lg.NctcanStoreBo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.vo.lg.NctcanStoreVo;
import org.mapstruct.*;
import org.mapstruct.factory.Mappers;


/**
 * Gitame Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring", uses = {
        IGctMapper.class, IGitMapper.class,
        INitMapper.class, INctMapper.class, IOtherModMapper.class}
)
public interface NctcanStructMapper extends IBaseStructMapper<NctcanStoreBo, NctcanStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    NctcanStructMapper INSTANCE = Mappers.getMapper(NctcanStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    NctcanStoreBo voTransferToBo(NctcanStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     *
     * @param vo
     * @param bo
     */
    @Override
    @Mapping(target = "nidgrp", ignore = true)
    @Mapping(target = "ncdgrp", ignore = true)
    void voUpdateToBo(NctcanStoreVo vo, @MappingTarget NctcanStoreBo bo);

    @AfterMapping
    default void afterUpdate(NctcanStoreVo vo, @MappingTarget NctcanStoreBo bo) {
        bo.getNcdgrp().getRec().setClsdat(vo.getNcdgrp().getRec().getClsdat());
        bo.setRebkpflg(vo.getRebkpflg());
    }

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    NctcanStoreVo boTransferToVo(NctcanStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(NctcanStoreBo bo, @MappingTarget NctcanStoreVo vo);
}
