package com.brilliance.isc.transfer.transaction;

import com.brilliance.isc.bo.Nit;
import com.brilliance.isc.bo.lg.NitcanStoreBo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.vo.lg.NitcanStoreVo;
import org.mapstruct.*;
import org.mapstruct.factory.Mappers;


/**
 * Gitame Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring", uses = {
        IGctMapper.class, IGitMapper.class,
        INitMapper.class, INctMapper.class, IOtherModMapper.class}
)
public interface NitcanStructMapper extends IBaseStructMapper<NitcanStoreBo, NitcanStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    NitcanStructMapper INSTANCE = Mappers.getMapper(NitcanStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    NitcanStoreBo voTransferToBo(NitcanStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     *
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({@Mapping(target = "nidgrp", ignore = true)})
    void voUpdateToBo(NitcanStoreVo vo, @MappingTarget NitcanStoreBo bo);

    @AfterMapping
    default void afterUpdate(NitcanStoreVo vo, @MappingTarget NitcanStoreBo bo) {
        bo.getNidgrp().getRec().setPurcan(vo.getNidgrp().getRec().getPurcan());
        bo.getNidgrp().getRec().setClsdat(vo.getNidgrp().getRec().getClsdat());
        bo.getNidgrp().getRec().setReddat(vo.getNidgrp().getRec().getReddat());
        bo.getNidgrp().getRec().setRedcur(vo.getNidgrp().getRec().getRedcur());
        bo.getNidgrp().getRec().setRedamt(vo.getNidgrp().getRec().getRedamt());
        bo.getNidgrp().getRec().setOutcur(vo.getNidgrp().getRec().getOutcur());
        bo.getNidgrp().getRec().setOutamt(vo.getNidgrp().getRec().getOutamt());
        Nit nit = bo.getNidgrp().getBlk();
        if (nit != null) {
            nit.setAccspc(vo.getNidgrp().getBlk().getAccspc());
            nit.setChaded(vo.getNidgrp().getBlk().getChaded());
            nit.setAmtspc(vo.getNidgrp().getBlk().getAmtspc());
        }
    }

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    NitcanStoreVo boTransferToVo(NitcanStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(NitcanStoreBo bo, @MappingTarget NitcanStoreVo vo);
}
